@@ -37,7 +37,7 @@ int drive( const std::string& dataFile, const std::string& mrfGFile, const std::
 			unsigned int nIter, unsigned int burnin, unsigned int nChains,
 			const std::string& covariancePrior, 
 			const std::string& gammaPrior, const std::string& gammaSampler, const std::string& gammaInit,
-			const std::string& betaPrior, const int maxThreads,
+			const std::string& betaPrior, const int maxThreads, const int tick,
 			bool output_gamma, bool output_beta, bool output_Gy, bool output_sigmaRho, bool output_pi, bool output_tail, bool output_model_size,
             bool output_CPO, bool output_model_visit );
 
